@@ -40,6 +40,7 @@ static gboolean cb_hci_data (BtctlDiscoverySource *source, gpointer user_data);
 
 static void name_request (int dd, bdaddr_t bdaddr);
 static void inquiry_result(int dd, unsigned char *buf, int len);
+static void inquiry_result_with_rssi(int dd, unsigned char *buf, int len);
 static void conn_request(int dd, unsigned char *buf, int len);
 static void conn_complete(int dd, unsigned char *buf, int len);
 static void name_complete(int dd, unsigned char *buf, int len);
@@ -67,6 +68,28 @@ inquiry_result(int dd, unsigned char *buf, int len)
 	}
 }
 
+static void
+inquiry_result_with_rssi(int dd, unsigned char *buf, int len)
+{
+	inquiry_info_with_rssi *info;
+	uint8_t num;
+	char addr[18];
+	int i;
+	uint32_t devclass;
+
+	num = buf[0];
+	printf("inquiry_result_with_rssi:\tnum %d\n", num);
+
+	for (i = 0; i < num; i++) {
+		info = (void *) (buf + (sizeof(*info) * i) + 1);
+		ba2str(&info->bdaddr, addr);
+        devclass = (uint32_t)(info->dev_class[2]) << 16 |
+            (uint32_t)(info->dev_class[1]) << 8 |
+            (uint32_t)(info->dev_class[0]);
+		printf("inquiry_result_with_rssi:\tbdaddr %s class %x\n", addr, devclass);
+	}
+}
+
 static void
 conn_request(int dd, unsigned char *buf, int len)
 {
@@ -160,6 +183,9 @@ cb_hci_data (BtctlDiscoverySource *source, gpointer user_data)
 	case EVT_INQUIRY_RESULT:
 		inquiry_result(source->fd.fd, ptr, len);
 		break;
+	case EVT_INQUIRY_RESULT_WITH_RSSI:
+		inquiry_result_with_rssi(source->fd.fd, ptr, len);
+		break;
 	case EVT_INQUIRY_COMPLETE:
 		inquiry_complete(source->fd.fd);
 		break;
