@@ -65,11 +65,11 @@ struct report_context {
 	guint chan;
 };
 
-typedef int (*handler_t) (BtctlController *bc, bdaddr_t *bdaddr, struct search_context *arg);
+typedef int (*handler_t) (BtctlController *bc, bdaddr_t *bdaddr, struct search_context *arg, GError **err);
 
-static int do_browse(BtctlController *bc, bdaddr_t *bdaddr, void *arg);
+static int do_browse(BtctlController *bc, bdaddr_t *bdaddr, void *arg, GError **err);
 
-static int do_search(BtctlController *bc, bdaddr_t *bdaddr, struct search_context *context);
+static int do_search(BtctlController *bc, bdaddr_t *bdaddr, struct search_context *context, GError **err);
 
 void fire_got_device(char *devaddr);
 void fire_got_device_name(char *devaddr, char *name);
@@ -80,6 +80,8 @@ static gboolean cb_hci_data (BtctlDiscoverySource *source, gpointer user_data);
 static void name_request (BtctlController *bc, bdaddr_t *bdaddr);
 static void inquiry_result (BtctlController *bc, int dd, unsigned char *buf,
         int len);
+static void inquiry_result_with_rssi (BtctlController *bc, int dd,
+	unsigned char *buf, int len);
 static void conn_request (BtctlController *bc, int dd, unsigned char *buf,
         int len);
 static void conn_complete (BtctlController *bc, int dd, unsigned char *buf,
@@ -272,7 +274,7 @@ print_service_attr(BtctlController *bc, char *id, sdp_record_t *rec)
 */
 
 void
-btctl_controller_impl_cmd_scan(BtctlController *bc)
+btctl_controller_impl_cmd_scan(BtctlController *bc, GError **err)
 {
 	inquiry_info *info=NULL;
 	int num_rsp, length, flags;
@@ -328,8 +330,8 @@ btctl_controller_impl_cmd_scan(BtctlController *bc)
 	if (dd>=0) {
 		for (i = 0; i < num_rsp; i++) {
 			memset(name, 0, sizeof(name));
-			if (hci_remote_name(dd, &(info+i)->bdaddr,
-						sizeof(name), name, 100000) < 0)
+			if (hci_read_remote_name(dd, &(info+i)->bdaddr,
+						 sizeof(name), name, 100000) < 0)
 				strcpy(name, "n/a");
 
 			baswap(&bdaddr, &(info+i)->bdaddr);
@@ -353,7 +355,7 @@ btctl_controller_impl_cmd_scan(BtctlController *bc)
 				free(info);
 				return;
 			}
-			do_browse(bc, &((info+i)->bdaddr), NULL);
+			do_browse(bc, &((info+i)->bdaddr), NULL, err);
 		}
 	} else {
 		perror("HCI device open failed.");
@@ -370,7 +372,7 @@ btctl_controller_impl_cmd_scan(BtctlController *bc)
  * Perform an inquiry and search/browse all peer found.
  */
 static void 
-inquiry(BtctlController *bc, handler_t handler, void *arg)
+inquiry(BtctlController *bc, handler_t handler, void *arg, GError **err)
 {
 	inquiry_info ii[20];
 	uint8_t count = 0;
@@ -382,7 +384,7 @@ inquiry(BtctlController *bc, handler_t handler, void *arg)
 		return;
 	}
 	for (i=0; i<count; i++) {
-		handler(bc, &ii[i].bdaddr, arg);
+		handler(bc, &ii[i].bdaddr, arg, err);
 	}
 }
 
@@ -391,7 +393,7 @@ inquiry(BtctlController *bc, handler_t handler, void *arg)
  * Search for a specific SDP service
  */
 static int 
-do_search(BtctlController *bc, bdaddr_t *bdaddr, struct search_context *context)
+do_search(BtctlController *bc, bdaddr_t *bdaddr, struct search_context *context, GError **err)
 {
 	sdp_list_t *attrid, *search, *seq, *next;
 	uint32_t range = 0x0000ffff;
@@ -400,7 +402,7 @@ do_search(BtctlController *bc, bdaddr_t *bdaddr, struct search_context *context)
 	int retries=2;
 
 	if (!bdaddr) {
-		inquiry(bc, do_search, context);
+		inquiry(bc, do_search, context, err);
 		return 0;
 	}
 	while (!sess && retries > 0) {
@@ -416,7 +418,11 @@ do_search(BtctlController *bc, bdaddr_t *bdaddr, struct search_context *context)
 
 	ba2str(bdaddr, str);
 	if (!sess) {
-		printf("Failed to connect to SDP server on %s\n", str);
+		if (err && *err) {
+			g_set_error (err, BTCTL_ERROR,
+			     BTCTL_ERROR_CONNECTION_FAILED,
+			     _("Bluetooth connection to remote device failed."));
+		}
 		return -1;
 	}
 	if (context->svc)
@@ -427,7 +433,11 @@ do_search(BtctlController *bc, bdaddr_t *bdaddr, struct search_context *context)
 	attrid = sdp_list_append(0, &range);
 	search = sdp_list_append(0, &context->group);
 	if (sdp_service_search_attr_req(sess, search, SDP_ATTR_REQ_RANGE, attrid, &seq)) {
-		printf("Service Search failed\n");
+		if (*err) {
+			g_set_error (err, BTCTL_ERROR,
+			     BTCTL_ERROR_SERVICE_SEARCH_FAILED,
+			     g_strdup_printf(_("Service search failed on device %s"), str));
+		}
 		sdp_close(sess);
 		return -1;
 	}
@@ -456,7 +466,7 @@ do_search(BtctlController *bc, bdaddr_t *bdaddr, struct search_context *context)
 			memcpy(&sub_context, context, sizeof(struct search_context));
 			/* Browse the next level down if not done */
 			if (sub_context.group.value.uuid16 != context->group.value.uuid16)
-				do_search(bc, bdaddr, &sub_context);
+				do_search(bc, bdaddr, &sub_context, err);
 		}
 		next = seq->next;
 		free(seq);
@@ -491,8 +501,8 @@ static guint brute_force_ids[] = {
 };
 
 
-static int 
-do_browse(BtctlController *bc, bdaddr_t *bdaddr, void *arg)
+static gboolean
+do_browse(BtctlController *bc, bdaddr_t *bdaddr, void *arg, GError **err)
 {
 	struct search_context context;
 	int i;
@@ -502,13 +512,13 @@ do_browse(BtctlController *bc, bdaddr_t *bdaddr, void *arg)
 	/* We want to browse the top-level/root  */
 	sdp_uuid16_create(&(context.group), PUBLIC_BROWSE_GROUP);
 	if (bdaddr)  {
-		ret = do_search(bc, bdaddr, &context);
+		ret = do_search(bc, bdaddr, &context, err);
 	} else {
-		ret = do_search(bc, 0, &context);
+		ret = do_search(bc, 0, &context, err);
 	}
 
 	if (ret < 0)
-		return ret;
+		return FALSE;
 
 	if (context.found == 0) {
 		/* Nothing found, so we'll brute-force search instead,
@@ -517,13 +527,15 @@ do_browse(BtctlController *bc, bdaddr_t *bdaddr, void *arg)
 		for (i=0; brute_force_ids[i]; i++) {
 			sdp_uuid16_create(&(context.group), brute_force_ids[i]);
 			if (bdaddr)  {
-				do_search(bc, bdaddr, &context);
+				if (do_search(bc, bdaddr, &context, err) < 0)
+					return FALSE;
 			} else {
-				do_search(bc, 0, &context);
+				if (do_search(bc, 0, &context, err) < 0)
+					return FALSE;
 			}
 		}
 	}
-	return 1;
+	return TRUE;
 }
 
 static char *rfcomm_state[] = {
@@ -773,7 +785,7 @@ btctl_controller_impl_list_rfcomm_connections(BtctlController *bc)
 /* lookup what channel a particular service is on */
 gint
 btctl_controller_impl_scan_for_service(BtctlController *bc,
-        const gchar *bdstr, guint clsid)
+        const gchar *bdstr, guint clsid, GError **err)
 {
 	bdaddr_t bdaddr;
 	struct search_context context;
@@ -782,7 +794,7 @@ btctl_controller_impl_scan_for_service(BtctlController *bc,
 	memset(&context, '\0', sizeof(struct search_context));
 	sdp_uuid16_create(&(context.group), clsid);
 
-	return (gint) do_search(bc, &bdaddr, &context);
+	return (gint) do_search(bc, &bdaddr, &context, err);
 }
 
 /* Find the signal strength for a particular device */
@@ -897,6 +909,36 @@ inquiry_result(BtctlController *bc, int dd, unsigned char *buf, int len)
 	}
 }
 
+static void
+inquiry_result_with_rssi(BtctlController *bc, int dd, unsigned char *buf, int len)
+{
+	inquiry_info_with_rssi *info;
+	uint8_t num;
+	char addr[18];
+	int i;
+	uint32_t devclass;
+	bdaddr_t *nb;
+
+	num = buf[0];
+
+	for (i = 0; i < num; i++) {
+		info = (void *) (buf + (sizeof(*info) * i) + 1);
+		ba2str(&info->bdaddr, addr);
+		devclass = (uint32_t)(info->dev_class[2]) << 16 |
+			(uint32_t)(info->dev_class[1]) << 8 |
+			(uint32_t)(info->dev_class[0]);
+		g_message ("inquiry_result_with_rssi:\tbdaddr %s class %x", addr, devclass);
+		nb = g_new0(bdaddr_t, 1);
+		bacpy(nb, &info->bdaddr);
+		/* inquiry can generate multiple identical results, so make
+		 * sure we only log each once */
+		if (! g_slist_find_custom (bc->found, (gpointer)nb, bdaddr_comparator)) {
+			bc->found = g_slist_append (bc->found, (gpointer)nb);
+			btctl_controller_got_device (bc, addr, devclass);
+		}
+	}
+}
+
 static void
 conn_request(BtctlController *bc, int dd, unsigned char *buf, int len)
 {
@@ -1019,6 +1061,9 @@ cb_hci_data (BtctlDiscoverySource *source, gpointer user_data)
 	case EVT_INQUIRY_RESULT:
 		inquiry_result(bc, source->fd.fd, ptr, len);
 		break;
+	case EVT_INQUIRY_RESULT_WITH_RSSI:
+		inquiry_result_with_rssi(bc, source->fd.fd, ptr,len);
+		break;
 	case EVT_INQUIRY_COMPLETE:
 		inquiry_complete(bc, source->fd.fd);
 		break;
@@ -1046,7 +1091,7 @@ btctl_controller_impl_get_discoverable(BtctlController *bc, GError **err)
 	if ((ctl = socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_HCI)) < 0) {
 		g_set_error (err, btctl_error_quark(),
 			     BTCTL_ERROR_NO_BLUETOOTH_DEVICE,
-			     "Can't open HCI socket: %s (%s)",
+			     "Can't open HCI socket: %s (%d)",
 			     strerror(errno), errno);
 		return FALSE;
 	}
@@ -1077,7 +1122,7 @@ btctl_controller_impl_set_discoverable(BtctlController *bc,
 	if ((ctl = socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_HCI)) < 0) {
 		g_set_error (err, btctl_error_quark(),
 			     BTCTL_ERROR_NO_BLUETOOTH_DEVICE,
-			     "Can't open HCI socket: %s (%s)",
+			     "Can't open HCI socket: %s (%d)",
 			     strerror(errno), errno);
 		return;
 	}
